==> kom%2B%2B-0.2.tar.gz: kom++-0.2/libisc/src/isc_alloc.c <==
/*
** isc_alloc.c                               ISC storage allocation routines
**
** Copyright (c) 1991 Peter Eriksson and Per Cederqvist of the
**                    Lysator Academic Computer Association.
**
** history:
** 910305 pen      moved into separate file
*/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include "isc.h"
#include "intern.h"

/*
** Some magic numbers
*/
#define ISC_MAGIC_ALLOC	0x12F54ACE
#define ISC_MAGIC_FREE  0xEE47A37F


/*
** External declarations
*/
#if defined(__sun__) && !defined(__stdlib_h)
extern void *malloc(size_t);
extern void *calloc(int, size_t);
extern void *realloc(void *, size_t);
extern void free(void *);
#endif

/*
** Pointers to functions to handle storage allocation
*/
static void * (*isc_mallocfn)(size_t size) =
    (void * (*)(size_t)) NULL;
  
static void * (*isc_reallocfn)(void *bufp, size_t size) =
    (void * (*)(void *, size_t)) NULL;
     
static void (*isc_freefn)(void *bufp) =
     (void (*)(void *)) NULL;


/*
** Define functions to handle storage allocation
*/
void
isc_setallocfn(void * (*mallocfn)(size_t size),
             void * (*reallocfn)(void *buf, size_t size),
	     void   (*freefn)(void *buf))
{
  isc_mallocfn  = mallocfn;
  isc_reallocfn = reallocfn;
  isc_freefn    = freefn;
}



void *
isc_malloc(size_t size)
{
  unsigned int  * buf;


  if (isc_mallocfn)
    buf = (unsigned int *) (*isc_mallocfn)(size + sizeof(unsigned int));
  else
  {
    buf = (unsigned int *) malloc(size + sizeof(unsigned int));
    if (!buf)
      isc_abort("isc_malloc");
  }

  *buf++ = ISC_MAGIC_ALLOC;

  return (void *) buf; 
}


void *
isc_realloc(void *oldbuf, size_t size)
{
  unsigned int *buf;


  if (!oldbuf)
    return isc_malloc(size);

  buf = (unsigned int *) oldbuf;
  switch (*--buf)
  {
    case ISC_MAGIC_ALLOC:
      break;

    case ISC_MAGIC_FREE:
      isc_abort("isc_realloc");

    default:
      isc_abort("isc_realloc");
  }

  *buf = ISC_MAGIC_FREE;
  
  if (isc_reallocfn)
    buf = (unsigned int *) (*isc_reallocfn)((void *) buf,
					    size + sizeof(unsigned int));
  else
  {
    buf = (void *) realloc((void *) buf, size + sizeof(unsigned int));
    if (!buf)
      isc_abort("isc_realloc");
  }

  *buf++ = ISC_MAGIC_ALLOC;
  return (void *) buf;
}



void
isc_free(void *buf)
{
  unsigned int *ibuf;

  
  if (!buf)
    isc_abort("isc_free");
    
  ibuf = (unsigned int *) buf;
  switch (*--ibuf)
  {
    case ISC_MAGIC_ALLOC:
      break;

    case ISC_MAGIC_FREE:
      isc_abort("isc_free");

    default:
      isc_abort("isc_free");
  }

  *ibuf = ISC_MAGIC_FREE;
  
  if (isc_freefn)
  {
    (*isc_freefn)((void *) ibuf);
    return;
  }
  else
    free((void *) ibuf);
}



/*
** Safe (hopefully) string duplicator
*/
char *
isc_strdup(const char *str)
{
  char *newstr;

  
  newstr = isc_malloc(strlen(str)+1);
  strcpy(newstr, str);
  return newstr;
}

==> kom%2B%2B-0.5.tar.gz: kom++-0.5/libisc/src/isc_alloc.c <==
/*
** isc_alloc.c                               ISC storage allocation routines
**
** Copyright (c) 1991, 1992, 1993 Peter Eriksson and Per Cederqvist of the
**                    Lysator Academic Computer Association.
**
** history:
** 910305 pen      moved into separate file
*/

#include <sys/types.h>
#include <sys/socket.h>
#ifdef HAVE_STDLIB_H
#  include <stdlib.h>
#endif
#ifdef HAVE_STDDEF_H
#  include <stddef.h>
#endif
#ifdef HAVE_STDARG_H
#  include <stdarg.h>
#endif
#ifdef HAVE_STRING_H
#  include <string.h>
#else
#  include <strings.h>
#endif
#ifndef NULL
#  include <stdio.h>
#endif

#include "isc.h"
#include "intern.h"

/*
** Some magic numbers
*/
#define ISC_MAGIC_ALLOC	0x12F54ACE
#define ISC_MAGIC_FREE  0xEE47A37F


/*
** External declarations
*/
#if defined(__sun__) && !defined(__stdlib_h)
extern void *malloc(size_t);
/*extern void *calloc(int, size_t);*/
extern void *realloc(void *, size_t);
extern void free(void *);
#endif

/*
** Pointers to functions to handle storage allocation
*/
static void * (*isc_mallocfn)(size_t size) = NULL;
static void * (*isc_reallocfn)(void *bufp, size_t size) = NULL;
static void (*isc_freefn)(void *bufp) = NULL;


/*
** Define functions to handle storage allocation
*/
void
isc_setallocfn(void * (*mallocfn)(size_t size),
             void * (*reallocfn)(void *buf, size_t size),
	     void   (*freefn)(void *buf))
{
  isc_mallocfn  = mallocfn;
  isc_reallocfn = reallocfn;
  isc_freefn    = freefn;
}



void *
isc_malloc(size_t size)
{
  unsigned long  * buf;


  if (isc_mallocfn)
    buf = (unsigned long *) (*isc_mallocfn)(size + sizeof(unsigned long));
  else
  {
    buf = (unsigned long *) malloc(size + sizeof(unsigned long));
    if (!buf)
      isc_abort("isc_malloc");
  }

  *buf++ = ISC_MAGIC_ALLOC;

  return (void *) buf; 
}


void *
isc_realloc(void *oldbuf, size_t size)
{
  unsigned long *buf;


  if (!oldbuf)
    return isc_malloc(size);

  buf = (unsigned long *) oldbuf;
  switch (*--buf)
  {
    case ISC_MAGIC_ALLOC:
      break;

    case ISC_MAGIC_FREE:
      isc_abort("isc_realloc");

    default:
      isc_abort("isc_realloc");
  }

  *buf = ISC_MAGIC_FREE;
  
  if (isc_reallocfn)
    buf = (unsigned long *) (*isc_reallocfn)((void *) buf,
					    size + sizeof(unsigned long));
  else
  {
    buf = (void *) realloc((void *) buf, size + sizeof(unsigned long));
    if (!buf)
      isc_abort("isc_realloc");
  }

  *buf++ = ISC_MAGIC_ALLOC;
  return (void *) buf;
}



void
isc_free(void *buf)
{
  unsigned long *ibuf;

  
  if (!buf)
    isc_abort("isc_free");
    
  ibuf = (unsigned long *) buf;
  switch (*--ibuf)
  {
    case ISC_MAGIC_ALLOC:
      break;

    case ISC_MAGIC_FREE:
      isc_abort("isc_free");

    default:
      isc_abort("isc_free");
  }

  *ibuf = ISC_MAGIC_FREE;
  
  if (isc_freefn)
  {
    (*isc_freefn)((void *) ibuf);
    return;
  }
  else
    free((void *) ibuf);
}



/*
** Safe (hopefully) string duplicator
*/
char *
isc_strdup(const char *str)
{
  char *newstr;

  
  newstr = isc_malloc(strlen(str)+1);
  strcpy(newstr, str);
  return newstr;
}
